"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright (c) 2018, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */
const vscode_1 = require("vscode");
const _ = require("lodash");
const renderer_1 = require("./renderer");
const messages_1 = require("../core/messages");
const utils_1 = require("../utils");
const path_1 = require("path");
const fs_1 = require("fs");
const DEBOUNCE_WAIT = 500;
const CMD_SHOW_API_EDITOR = "ballerina.showAPIEditor";
let oasEditorPanel;
let activeEditor;
let preventAPIDesignerUpdate = false;
function updateOASWebView(docUri, resp, stale) {
    if (oasEditorPanel) {
        oasEditorPanel.webview.postMessage({
            command: 'update',
            docUri: docUri.toString(),
            json: resp,
            stale
        });
    }
}
function showAPIEditorPanel(context, langClient, serviceName) {
    vscode_1.workspace.onDidChangeTextDocument(_.debounce((e) => {
        if (activeEditor && (e.document === activeEditor.document) &&
            e.document.fileName.endsWith('.bal')) {
            if (preventAPIDesignerUpdate) {
                preventAPIDesignerUpdate = false;
                return;
            }
            const docUri = e.document.uri;
            if (oasEditorPanel) {
                langClient.getBallerinaOASDef(docUri, oasEditorPanel.title.split('-')[1].trim()).then((resp) => {
                    if (resp.ballerinaOASJson !== undefined) {
                        updateOASWebView(docUri, JSON.stringify(resp.ballerinaOASJson), false);
                        preventAPIDesignerUpdate = true;
                    }
                });
            }
        }
    }, DEBOUNCE_WAIT));
    vscode_1.window.onDidChangeActiveTextEditor((activatedEditor) => {
        if (vscode_1.window.activeTextEditor && activatedEditor
            && (activatedEditor.document === vscode_1.window.activeTextEditor.document)
            && activatedEditor.document.fileName.endsWith('.bal')
            && activatedEditor !== activeEditor) {
            activeEditor = vscode_1.window.activeTextEditor;
            if (oasEditorPanel) {
                langClient.getServiceListForActiveFile(activeEditor.document.uri).then((resp) => {
                    if (resp.services && resp.services.length > 1) {
                        vscode_1.window.showQuickPick(resp.services).then((selected) => {
                            if (selected && activeEditor) {
                                const html = renderer_1.apiEditorRender(context, langClient, activeEditor.document.uri, selected);
                                if (oasEditorPanel && html) {
                                    oasEditorPanel.webview.html = html;
                                    oasEditorPanel.title = "Ballerina API Designer - " + selected;
                                }
                            }
                        });
                    }
                    else {
                        if (activeEditor) {
                            const html = renderer_1.apiEditorRender(context, langClient, activeEditor.document.uri, resp.services[0]);
                            if (oasEditorPanel && html) {
                                oasEditorPanel.webview.html = html;
                                oasEditorPanel.title = "Ballerina API Designer - " + resp.services[0];
                            }
                        }
                    }
                });
            }
        }
    });
    const editor = vscode_1.window.activeTextEditor;
    // TODO : proper handler if not the active editor
    if (!editor) {
        return "";
    }
    activeEditor = editor;
    let executeCreateAPIEditor = function (serviceName) {
        let renderHtml = renderer_1.apiEditorRender(context, langClient, editor.document.uri, serviceName);
        createAPIEditorPanel(serviceName, renderHtml, langClient, context);
    };
    if (serviceName) {
        executeCreateAPIEditor(serviceName);
    }
    else {
        langClient.getServiceListForActiveFile(activeEditor.document.uri).then(resp => {
            if (resp.services.length === 0) {
                const actions = [];
                // Provide an action to fill up empty bal files with a default service
                const actionAddService = "Add HTTP Service";
                if (activeEditor && activeEditor.document.getText().trim().length === 0) {
                    actions.push(actionAddService);
                }
                vscode_1.window.showInformationMessage(messages_1.API_DESIGNER_NO_SERVICE, ...actions)
                    .then((selection) => {
                    if (selection === actionAddService) {
                        const svcTemplatePath = path_1.join(context.extensionPath, "resources", "templates", "http-service.bal");
                        const svcTemplate = fs_1.readFileSync(svcTemplatePath).toString();
                        if (activeEditor) {
                            activeEditor.edit((editBuilder) => {
                                editBuilder.insert(new vscode_1.Position(0, 0), svcTemplate);
                            }).then((insertSuccess) => {
                                if (insertSuccess) {
                                    vscode_1.commands.executeCommand(CMD_SHOW_API_EDITOR);
                                }
                            });
                        }
                    }
                });
            }
            else if (resp.services && resp.services.length > 1) {
                vscode_1.window.showQuickPick(resp.services).then(service => {
                    if (service && activeEditor) {
                        executeCreateAPIEditor(service);
                    }
                });
            }
            else {
                executeCreateAPIEditor(resp.services[0]);
            }
        });
    }
}
function createAPIEditorPanel(selectedService, renderHtml, langClient, context) {
    if (!oasEditorPanel) {
        oasEditorPanel = vscode_1.window.createWebviewPanel('ballerinaOASEditor', 'Ballerina API Designer - ' + selectedService, { viewColumn: vscode_1.ViewColumn.Two, preserveFocus: true }, {
            enableScripts: true,
            retainContextWhenHidden: true,
        });
    }
    oasEditorPanel.webview.html = renderHtml;
    const remoteMethods = [
        {
            methodName: "getOpenApiDef",
            handler: (args) => {
                return langClient.getBallerinaOASDef(args[0], args[1]);
            }
        }, {
            methodName: 'triggerOpenApiDefChange',
            handler: (args) => {
                return langClient.triggerOpenApiDefChange(args[0], args[1]);
            }
        }
    ];
    utils_1.WebViewRPCHandler.create(oasEditorPanel.webview, langClient, remoteMethods);
    oasEditorPanel.iconPath = {
        light: vscode_1.Uri.file(path_1.join(context.extensionPath, 'resources/images/icons/api-design.svg')),
        dark: vscode_1.Uri.file(path_1.join(context.extensionPath, 'resources/images/icons/api-design-inverse.svg'))
    };
    oasEditorPanel.webview.onDidReceiveMessage(message => {
        switch (message.command) {
            case 'oasASTModified':
                if (activeEditor && activeEditor.document.fileName.endsWith('.bal')) {
                    preventAPIDesignerUpdate = true;
                }
                return;
        }
    }, undefined, context.subscriptions);
    oasEditorPanel.onDidDispose(() => {
        oasEditorPanel = undefined;
    });
}
function activate(ballerinaExtInstance) {
    let context = ballerinaExtInstance.context;
    let langClient = ballerinaExtInstance.langClient;
    const showAPIRenderer = vscode_1.commands.registerCommand(CMD_SHOW_API_EDITOR, serviceNameArg => {
        ballerinaExtInstance.onReady()
            .then(() => {
            const { experimental } = langClient.initializeResult.capabilities;
            const serverProvidesAPIEditorFeature = experimental && experimental.apiEditorProvider;
            if (!serverProvidesAPIEditorFeature) {
                ballerinaExtInstance.showMessageServerMissingCapability();
                return;
            }
            showAPIEditorPanel(context, langClient, (serviceNameArg) ? serviceNameArg.argumentV : "");
        })
            .catch((e) => {
            if (!ballerinaExtInstance.isValidBallerinaHome()) {
                ballerinaExtInstance.showMessageInvalidBallerinaHome();
            }
            else {
                ballerinaExtInstance.showPluginActivationError();
            }
        });
    });
    context.subscriptions.push(showAPIRenderer);
}
exports.activate = activate;
//# sourceMappingURL=activator.js.map