'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright (c) 2018, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */
const vscode_1 = require("vscode");
const messages_1 = require("./messages");
const path = require("path");
const fs = require("fs");
const child_process_1 = require("child_process");
const vscode_languageclient_1 = require("vscode-languageclient");
const server_1 = require("../server/server");
const extended_language_client_1 = require("./extended-language-client");
const index_1 = require("../utils/index");
const assert_1 = require("assert");
class BallerinaExtension {
    constructor() {
        this.ballerinaHome = '';
        // Load the extention
        this.extention = vscode_1.extensions.getExtension('ballerina.ballerina');
        this.clientOptions = {
            documentSelector: [{ scheme: 'file', language: 'ballerina' }],
            outputChannel: index_1.getOutputChannel(),
            revealOutputChannelOn: vscode_languageclient_1.RevealOutputChannelOn.Never,
        };
    }
    setContext(context) {
        this.context = context;
    }
    init(onBeforeInit) {
        try {
            // Register pre init handlers.
            this.registerPreInitHandlers();
            // Check if ballerina home is set.
            if (this.hasBallerinaHomeSetting()) {
                index_1.info("Ballerina home is configured in settings.");
                this.ballerinaHome = this.getBallerinaHome();
                // Lets check if ballerina home is valid.
                if (!this.isValidBallerinaHome(this.ballerinaHome)) {
                    index_1.info("Configured Ballerina home is not valid.");
                    // Ballerina home in setting is invalid show message and quit.
                    // Prompt to correct the home. // TODO add auto ditection.
                    this.showMessageInvalidBallerinaHome();
                    return Promise.resolve();
                }
            }
            else {
                index_1.info("Auto detecting Ballerina home.");
                // If ballerina home is not set try to auto ditect ballerina home.
                // TODO If possible try to update the setting page.
                this.ballerinaHome = this.autoDitectBallerinaHome();
                if (!this.ballerinaHome) {
                    this.showMessageInstallBallerina();
                    index_1.info("Unable to auto detect Ballerina home.");
                    return Promise.resolve();
                }
            }
            index_1.info("Using " + this.ballerinaHome + " as the Ballerina home.");
            // Validate the ballerina version.
            const pluginVersion = this.extention.packageJSON.version.split('-')[0];
            return this.getBallerinaVersion(this.ballerinaHome).then(ballerinaVersion => {
                ballerinaVersion = ballerinaVersion.split('-')[0];
                index_1.info(`Plugin version: ${pluginVersion}\nBallerina version: ${ballerinaVersion}`);
                this.checkCompatibleVersion(pluginVersion, ballerinaVersion);
                // if Home is found load Language Server.
                this.langClient = new extended_language_client_1.ExtendedLangClient('ballerina-vscode', 'Ballerina LS Client', server_1.getServerOptions(this.getBallerinaHome(), this.isExperimental()), this.clientOptions, false);
                // 0.983.0 and 0.982.0 versions are incable of handling client capabilies 
                if (ballerinaVersion !== "0.983.0" && ballerinaVersion !== "0.982.0") {
                    onBeforeInit(this.langClient);
                }
                // Following was put in to handle server startup failiers.
                const disposeDidChange = this.langClient.onDidChangeState(stateChangeEvent => {
                    if (stateChangeEvent.newState === vscode_languageclient_1.State.Stopped) {
                        index_1.info("Couldn't establish language server connection.");
                        this.showPluginActivationError();
                    }
                });
                let disposable = this.langClient.start();
                this.langClient.onReady().then(fullfilled => {
                    disposeDidChange.dispose();
                    this.context.subscriptions.push(disposable);
                });
            }).catch(e => {
                index_1.info(`Error when checking ballerina version. Got ${e}`);
            });
        }
        catch (ex) {
            index_1.info("Error while activating plugin: " + (ex.message ? ex.message : ex));
            // If any failure occurs while intializing show an error messege
            this.showPluginActivationError();
            return Promise.resolve();
        }
    }
    onReady() {
        if (!this.langClient) {
            return Promise.reject('BallerinaExtension is not initialized');
        }
        return this.langClient.onReady();
    }
    showPluginActivationError() {
        // message to display on Unknoen errors.
        // ask to enable debuglogs.
        // we can ask the user to report the issue.
        vscode_1.window.showErrorMessage(messages_1.UNKNOWN_ERROR);
    }
    registerPreInitHandlers() {
        // We need to restart VSCode if we change plugin configurations.
        vscode_1.workspace.onDidChangeConfiguration((params) => {
            if (params.affectsConfiguration('ballerina.home') ||
                params.affectsConfiguration('ballerina.allowExperimental') ||
                params.affectsConfiguration('ballerina.debugLog')) {
                this.showMsgAndRestart(messages_1.CONFIG_CHANGED);
            }
            if (params.affectsConfiguration('ballerina')) {
                const args = {
                    settings: vscode_1.workspace.getConfiguration('ballerina'),
                };
                this.langClient.sendNotification("workspace/didChangeConfiguration", args);
            }
        });
        vscode_1.languages.setLanguageConfiguration('ballerina', {
            onEnterRules: [
                {
                    beforeText: new RegExp('^\\s*#'),
                    action: {
                        appendText: '# ',
                        indentAction: vscode_1.IndentAction.None,
                    }
                }
            ]
        });
    }
    showMsgAndRestart(msg) {
        const action = 'Restart Now';
        vscode_1.window.showInformationMessage(msg, action).then((selection) => {
            if (action === selection) {
                vscode_1.commands.executeCommand('workbench.action.reloadWindow');
            }
        });
    }
    /**
     * Compares plugin's versions with the used ballerina distribution's version
     * Uses only the major and minor versions according to the semver spec.
     * First two numbers will be used when version string is not semver (eg. 0.990-r1)
     * Returns 1 if plugin version is higher than ballerina's; -1 if plugin version is lower
     * than ballerina's; 0 if the versions match.
     *
     * @returns {number}
     */
    compareVersions(pluginVersion, ballerinaVersion) {
        const toInt = (i) => {
            return parseInt(i, 10);
        };
        const numMatchRegexp = /\d+/g;
        const [pluginMajor, pluginMinor] = pluginVersion.match(numMatchRegexp).map(toInt);
        const [ballerinaMajor, ballerinaMinor] = ballerinaVersion.match(numMatchRegexp).map(toInt);
        if (pluginMajor > ballerinaMajor) {
            return 1;
        }
        if (pluginMajor < ballerinaMajor) {
            return -1;
        }
        if (pluginMinor > ballerinaMinor) {
            return 1;
        }
        if (pluginMinor < ballerinaMinor) {
            return -1;
        }
        return 0;
    }
    checkCompatibleVersion(pluginVersion, ballerinaVersion) {
        const versionCheck = this.compareVersions(pluginVersion, ballerinaVersion);
        if (versionCheck > 0) {
            // Plugin version is greater
            this.showMessageOldBallerina();
            return;
        }
        if (versionCheck < 0) {
            // Ballerina version is greater
            this.showMessageOldPlugin();
        }
    }
    getBallerinaVersion(ballerinaHome) {
        if (!ballerinaHome) {
            throw new assert_1.AssertionError({
                message: "Trying to get ballerina version without setting ballerina home."
            });
        }
        let command = `${path.join(ballerinaHome, 'bin', 'ballerina')} version`;
        if (process.platform === 'win32') {
            command = `"${path.join(ballerinaHome, 'bin', 'ballerina.bat')}" version`;
        }
        return new Promise((resolve, reject) => {
            child_process_1.exec(command, (err, stdout, stderr) => {
                const version = stdout.length > 0 ? stdout : stderr;
                if (version.startsWith("Error:")) {
                    reject(version);
                    return;
                }
                resolve(version.replace(/Ballerina /, '').replace(/[\n\t\r]/g, ''));
            });
        });
    }
    showMessageInstallBallerina() {
        const download = 'Download';
        const openSettings = 'Open Settings';
        vscode_1.window.showWarningMessage(messages_1.INSTALL_BALLERINA, download, openSettings).then((selection) => {
            if (openSettings === selection) {
                vscode_1.commands.executeCommand('workbench.action.openGlobalSettings');
            }
            if (download === selection) {
                vscode_1.commands.executeCommand('vscode.open', vscode_1.Uri.parse(messages_1.DOWNLOAD_BALLERINA));
            }
        });
    }
    showMessageInvalidBallerinaHome() {
        const action = 'Open Settings';
        vscode_1.window.showWarningMessage(messages_1.INVALID_HOME_MSG, action).then((selection) => {
            if (action === selection) {
                vscode_1.commands.executeCommand('workbench.action.openGlobalSettings');
            }
        });
    }
    showMessageOldBallerina() {
        const download = 'Download';
        vscode_1.window.showWarningMessage(messages_1.OLD_BALLERINA_VERSION, download).then((selection) => {
            if (download === selection) {
                vscode_1.commands.executeCommand('vscode.open', vscode_1.Uri.parse(messages_1.DOWNLOAD_BALLERINA));
            }
        });
    }
    showMessageOldPlugin() {
        const download = 'Download';
        vscode_1.window.showWarningMessage(messages_1.OLD_PLUGIN_VERSION, download).then((selection) => {
            if (download === selection) {
                vscode_1.commands.executeCommand('vscode.open', vscode_1.Uri.parse(messages_1.DOWNLOAD_BALLERINA));
            }
        });
    }
    showMessageServerMissingCapability() {
        const download = 'Download';
        vscode_1.window.showErrorMessage(messages_1.MISSING_SERVER_CAPABILITY, download).then((selection) => {
            if (download === selection) {
                vscode_1.commands.executeCommand('vscode.open', vscode_1.Uri.parse(messages_1.DOWNLOAD_BALLERINA));
            }
        });
    }
    showMessageInvalidFile() {
        vscode_1.window.showErrorMessage(messages_1.INVALID_FILE);
    }
    isValidBallerinaHome(homePath = this.ballerinaHome) {
        const ballerinaCmd = process.platform === 'win32' ? 'ballerina.bat' : 'ballerina';
        if (fs.existsSync(path.join(homePath, 'bin', ballerinaCmd))) {
            return true;
        }
        return false;
    }
    /**
     * Get ballerina home path.
     *
     * @returns {string}
     * @memberof BallerinaExtention
     */
    getBallerinaHome() {
        if (this.ballerinaHome) {
            return this.ballerinaHome;
        }
        else {
            return vscode_1.workspace.getConfiguration().get('ballerina.home');
        }
    }
    isExperimental() {
        return vscode_1.workspace.getConfiguration().get('ballerina.allowExperimental');
    }
    autoDitectBallerinaHome() {
        // try to ditect the environment.
        const platform = process.platform;
        let ballerinaPath = '';
        switch (platform) {
            case 'win32': // Windows
                if (process.env.BALLERINA_HOME) {
                    return process.env.BALLERINA_HOME;
                }
                try {
                    ballerinaPath = child_process_1.execSync('where ballerina').toString().trim();
                }
                catch (error) {
                    return ballerinaPath;
                }
                if (ballerinaPath) {
                    ballerinaPath = ballerinaPath.replace(/bin\\ballerina.bat$/, '');
                }
                break;
            case 'darwin': // Mac OS
                try {
                    const output = child_process_1.execSync('which ballerina');
                    ballerinaPath = fs.realpathSync(output.toString().trim());
                    // remove ballerina bin from ballerinaPath
                    if (ballerinaPath) {
                        ballerinaPath = ballerinaPath.replace(/bin\/ballerina$/, '');
                        // For homebrew installations ballerina executables are in libexcec
                        const homebrewBallerinaPath = path.join(ballerinaPath, 'libexec');
                        if (fs.existsSync(homebrewBallerinaPath)) {
                            ballerinaPath = homebrewBallerinaPath;
                        }
                    }
                }
                catch (_a) {
                    return ballerinaPath;
                }
                break;
            case 'linux': // Linux
                // lets see where the ballerina command is.
                try {
                    const output = child_process_1.execSync('which ballerina');
                    ballerinaPath = fs.realpathSync(output.toString().trim());
                    // remove ballerina bin from path
                    if (ballerinaPath) {
                        ballerinaPath = ballerinaPath.replace(/bin\/ballerina$/, '');
                    }
                }
                catch (_b) {
                    return ballerinaPath;
                }
                break;
        }
        // If we cannot find ballerina home return empty.
        return ballerinaPath;
    }
    hasBallerinaHomeSetting() {
        const home = this.getBallerinaHome();
        if (home) {
            return true;
        }
        return false;
    }
}
exports.BallerinaExtension = BallerinaExtension;
exports.ballerinaExtInstance = new BallerinaExtension();
//# sourceMappingURL=extension.js.map