"use strict";
/**
 * Copyright (c) 2018, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const WebSocket = require("ws");
// See http://tools.ietf.org/html/rfc6455#section-7.4.1
const WS_NORMAL_CODE = 1000;
const WS_SSL_CODE = 1015;
const WS_PING_INTERVAL = 15000;
/**
 * Handles websocket communitation with debugger backend
 * @class DebugChannel
 * @extends {EventEmitter}
 */
class DebugChannel extends events_1.EventEmitter {
    /**
     * Creates an instance of DebugChannel.
     * @param {string} endpoint - endpoint
     *
     * @memberof DebugChannel
     */
    constructor(endpoint) {
        super();
        if (!endpoint) {
            throw new Error('Invalid Endpoint');
        }
        this._endpoint = endpoint;
    }
    /**
     * Connect to debugger backend
     *
     * @memberof DebugChannel
     */
    connect() {
        const websocket = new WebSocket(this._endpoint);
        websocket.onmessage = (message) => { this.parseMessage(message.data); };
        websocket.onopen = (event) => { this.onOpen(event); };
        websocket.onclose = (event) => { this.onClose(event); };
        websocket.onerror = (event) => { this.onError(event); };
        this._websocket = websocket;
    }
    /**
     * Parses string to JSON
     * @param {WebSocket.Data} msgData - message
     *
     * @memberof DebugChannel
     */
    parseMessage(msgData) {
        const message = JSON.parse(msgData);
        this.emit('onmessage', message);
    }
    /**
     * Sends message to backend
     * @param {Object} message - object to send
     *
     * @memberof DebugChannel
     */
    sendMessage(message) {
        if (this._websocket) {
            this._websocket.send(JSON.stringify(message));
        }
    }
    /**
     * Handles websocket onClose event
     * @param {WebSocketOnCloseEvt} event - websocket onClose event
     *
     * @memberof DebugChannel
     */
    onClose(event) {
        this.stopPing();
        let reason;
        if (event.code === WS_NORMAL_CODE) {
            reason = 'Normal closure';
            this.emit('session-ended');
            return;
        }
        else if (event.code === WS_SSL_CODE) {
            reason = 'Certificate Issue';
        }
        else {
            reason = `Debug socket close with reason :${event.code}`;
        }
        this.emit('session-terminated', { reason });
    }
    /**
     * Handles websocket onError event
     *
     * @memberof DebugChannel
     */
    onError(event) {
        this.stopPing();
        this.emit('session-error', event, this._endpoint);
    }
    /**
     * Handles websocket onOpen event
     *  @param {WebSocketOnOpenEvt} event - websocket onOpen event
     *
     * @memberof DebugChannel
     */
    onOpen(event) {
        this.emit('connected');
        //this.startPing();
    }
    /**
     * start a ping for the websocket.
     *
     * @memberof LaunchDebugChannel
     */
    startPing() {
        this._ping = setInterval(() => {
            this.sendMessage({ command: 'PING' });
        }, WS_PING_INTERVAL);
    }
    stopPing() {
        if (this._ping) {
            clearInterval(this._ping);
        }
    }
    /**
     * Close websocket DebugChannel.
     *
     * @memberof LaunchDebugChannel
     */
    close() {
        this.stopPing();
        if (this._websocket) {
            this._websocket.close();
        }
        this.emit('connection-closed');
    }
}
exports.DebugChannel = DebugChannel;
//# sourceMappingURL=channel.js.map