"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright (c) 2018, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
const vscode_debugadapter_1 = require("vscode-debugadapter");
const ms_wmic_1 = require("ms-wmic");
const manager_1 = require("./manager");
const fs = require("fs");
const os = require("os");
const path = require("path");
const child_process_1 = require("child_process");
const openport_1 = require("openport");
const ps_node_1 = require("ps-node");
const toml = require("toml");
const IS_WIN = process.platform === 'win32';
class BallerinaDebugSession extends vscode_debugadapter_1.LoggingDebugSession {
    constructor() {
        super('ballerina-debug.txt');
        this._dirPaths = new Map();
        this._threadIndexes = new Map();
        this._nextThreadId = 1;
        this._nextFrameId = 1;
        this._nextVariableRefId = 1;
        this._threads = new Map();
        this._frames = new Map();
        this._variableRefs = new Map();
        this._debugManager = new manager_1.DebugManager();
        this._debugTests = false;
        this._executableArgs = [];
    }
    initializeRequest(response, args) {
        response.body = response.body || {};
        response.body.supportsConfigurationDoneRequest = true;
        this._debugManager.on('debug-hit', debugArgs => {
            const serverThreadId = debugArgs.threadId || debugArgs.workerId;
            if (!this._threadIndexes.get(serverThreadId)) {
                const thread = {
                    id: this._nextThreadId++,
                    name: serverThreadId.split('-')[0],
                    serverThreadId,
                    frameIds: [],
                };
                this._threads.set(thread.id, thread);
                this._threadIndexes.set(serverThreadId, thread.id);
            }
            // we can cast this as we are sure above exists
            const threadId = this._threadIndexes.get(serverThreadId);
            const threadObj = this._threads.get(threadId);
            // Clear other frames for this thread
            threadObj.frameIds.forEach(frameId => {
                const frame = this._frames.get(frameId);
                frame.scopes.forEach(scope => {
                    this._variableRefs.delete(scope.variablesReference);
                });
                this._frames.delete(frameId);
            });
            threadObj.frameIds = [];
            //TODO fix any type
            debugArgs.frames.forEach((frame) => {
                const { fileName, frameName, lineID, packageName: packageInfo } = frame;
                let packageNameParts = packageInfo.split(':')[0].split('/');
                const packageDirname = packageNameParts[1] || packageNameParts[0];
                const frameObj = {
                    threadId,
                    id: this._nextFrameId++,
                    scopes: new Array(),
                    fileName: path.join(packageDirname, fileName),
                    frameName,
                    line: lineID
                };
                ['Local', 'Global'].forEach(scopeName => {
                    const scope = {
                        name: scopeName,
                        variablesReference: this._nextVariableRefId++,
                        expensive: false,
                    };
                    const variables = frame.variables.filter((variable) => variable.scope === scopeName);
                    frameObj.scopes.push(scope);
                    this._variableRefs.set(scope.variablesReference, {
                        threadId,
                        variables,
                    });
                });
                threadObj.frameIds.push(frameObj.id);
                this._frames.set(frameObj.id, frameObj);
            });
            this.sendEvent(new vscode_debugadapter_1.StoppedEvent('breakpoint', this._threadIndexes.get(serverThreadId)));
        });
        this._debugManager.on('execution-ended', () => {
            this.sendEvent(new vscode_debugadapter_1.TerminatedEvent());
        });
        this._debugManager.on('session-error', e => {
            if (e) {
                this.sendEvent(new vscode_debugadapter_1.OutputEvent(e.message));
            }
        });
        this.sendResponse(response);
    }
    attachRequest(response, args) {
        const openFile = args.script;
        let cwd = path.dirname(openFile);
        this.setSourceRoot(cwd);
        const { sourceRoot, ballerinaPackage } = this._getRunningInfo(cwd, path.parse(openFile).root);
        if (ballerinaPackage && sourceRoot) {
            this._ballerinaPackage = ballerinaPackage;
            this.setSourceRoot(cwd);
            try {
                const balConfigString = fs.readFileSync(path.join(sourceRoot, 'Ballerina.toml'));
                this._projectConfig = toml.parse(balConfigString.toString()).project;
                this.setSourceRoot(sourceRoot);
            }
            catch (e) {
                console.log(e);
                // no log file
            }
        }
        else {
            this.setSourceRoot(cwd);
        }
        this._debugManager.connect(`ws://${args.host}:${args.port}/debug`, () => {
            this.sendResponse(response);
            this.sendEvent(new vscode_debugadapter_1.InitializedEvent());
        });
    }
    _getRunningInfo(currentPath, root, ballerinaPackage = undefined) {
        if (fs.existsSync(path.join(currentPath, '.ballerina'))) {
            if (currentPath !== os.homedir()) {
                return {
                    sourceRoot: currentPath,
                    ballerinaPackage,
                };
            }
        }
        if (currentPath === root) {
            return {};
        }
        return this._getRunningInfo(path.dirname(currentPath), root, path.basename(currentPath));
    }
    setSourceRoot(sourceRoot) {
        this._sourceRoot = sourceRoot;
    }
    launchRequest(response, args) {
        if (!args['ballerina.home']) {
            this.terminate("Couldn't start the debug server. Please set ballerina.home.");
            return;
        }
        this._noDebug = args.noDebug;
        const openFile = args.script;
        const scriptArguments = args.scriptArguments;
        const commandOptions = args.commandOptions;
        this._debugTests = args.debugTests;
        let cwd = path.dirname(openFile);
        let debugTarget = path.basename(openFile);
        this._sourceRoot = cwd;
        this._ballerinaPackage = '.';
        const { sourceRoot, ballerinaPackage } = this._getRunningInfo(cwd, path.parse(openFile).root);
        if (sourceRoot) {
            this.setSourceRoot(sourceRoot);
        }
        if (ballerinaPackage) {
            this._ballerinaPackage = ballerinaPackage;
            debugTarget = ballerinaPackage;
            cwd = sourceRoot;
            try {
                const balConfigString = fs.readFileSync(path.join(sourceRoot, 'Ballerina.toml'));
                this._projectConfig = toml.parse(balConfigString.toString()).project;
            }
            catch (e) {
                // no log file
            }
        }
        let executable = path.join(args['ballerina.home'], 'bin', 'ballerina');
        if (process.platform === 'win32') {
            executable += '.bat';
        }
        this._debugTarget = debugTarget;
        // find an open port
        openport_1.find((err, port) => {
            if (err) {
                this.terminate("Couldn't find an open port to start the debug server.");
                return;
            }
            this._debugPort = port.toString();
            let executableArgs = [this._debugTests ? "test" : "run"];
            executableArgs.push('--debug');
            executableArgs.push(this._debugPort);
            executableArgs.push('--experimental');
            if (Array.isArray(commandOptions) && commandOptions.length) {
                executableArgs = executableArgs.concat(commandOptions);
            }
            if (args.networkLogs && args.networkLogsPort > 0) {
                executableArgs.push('-e');
                executableArgs.push('b7a.http.tracelog.host=localhost');
                executableArgs.push('-e');
                executableArgs.push(`b7a.http.tracelog.port=${args.networkLogsPort}`);
            }
            executableArgs.push(this._debugTarget);
            if (Array.isArray(scriptArguments) && scriptArguments.length) {
                executableArgs = executableArgs.concat(scriptArguments);
            }
            this._executableArgs = executableArgs;
            let debugServer = this._debugServer = child_process_1.spawn(executable, executableArgs, { cwd });
            debugServer.on('error', (err) => {
                this.terminate("Could not start the debug server.");
            });
            debugServer.stdout.on('data', (data) => {
                if (`${data}`.indexOf('Ballerina remote debugger is activated on port') > -1) {
                    this._debugManager.connect(`ws://127.0.0.1:${port}/debug`, () => {
                        this.sendResponse(response);
                        this.sendEvent(new vscode_debugadapter_1.InitializedEvent());
                    });
                }
                this.sendEvent(new vscode_debugadapter_1.OutputEvent(`${data}`));
            });
            debugServer.stderr.on('data', (data) => {
                if (`${data}`.startsWith("error:")) {
                    this.terminate(`${data}`);
                }
                this.sendEvent(new vscode_debugadapter_1.OutputEvent(`${data}`));
            });
        });
    }
    setBreakPointsRequest(response, args) {
        if (args.source && args.source.path && args.source.name) {
            let fileName = args.source.path;
            let pkg = '.';
            const { _projectConfig, _ballerinaPackage } = this;
            if (_ballerinaPackage) {
                pkg = _ballerinaPackage;
            }
            if (_projectConfig) {
                pkg = `${_projectConfig['org-name']}/${_ballerinaPackage}:${_projectConfig.version}`;
            }
            this._dirPaths.set(args.source.name, path.dirname(args.source.path));
            this._debugManager.removeAllBreakpoints(fileName);
            const bps = [];
            if (!this._noDebug && args.breakpoints) {
                args.breakpoints.forEach((bp, i) => {
                    this._debugManager.addBreakPoint(bp.line, fileName, pkg);
                    bps.push({ id: i, line: bp.line, verified: true });
                });
            }
            response.body = {
                breakpoints: bps,
            };
            this.sendResponse(response);
        }
    }
    configurationDoneRequest(response, args) {
        this._debugManager.startDebug();
        this.sendResponse(response);
    }
    threadsRequest(response) {
        const threads = [];
        this._threads.forEach((thread) => {
            threads.push({ id: thread.id, name: thread.name });
        });
        response.body = { threads };
        this.sendResponse(response);
    }
    stackTraceRequest(response, args) {
        const thread = this._threads.get(args.threadId);
        if (thread) {
            const stk = thread.frameIds.map((frameId) => {
                const frame = this._frames.get(frameId);
                const filePath = path.join(this._sourceRoot, frame.fileName);
                return {
                    id: frameId,
                    name: frame.frameName,
                    line: frame.line,
                    column: 0,
                    source: {
                        name: frame.fileName,
                        path: filePath,
                        sourceReference: 0
                    }
                };
            });
            response.body = {
                stackFrames: stk,
                totalFrames: stk.length
            };
            this.sendResponse(response);
        }
    }
    scopesRequest(response, args) {
        const frame = this._frames.get(args.frameId);
        if (frame) {
            response.body = {
                scopes: frame.scopes,
            };
            this.sendResponse(response);
        }
    }
    variablesRequest(response, args) {
        const varRef = this._variableRefs.get(args.variablesReference);
        if (varRef) {
            const variables = varRef.variables;
            response.body = {
                variables: variables.map(variable => ({
                    name: variable.name,
                    type: "integer",
                    value: variable.value,
                    variablesReference: 0
                })),
            };
            this.sendResponse(response);
        }
    }
    continueRequest(response, args) {
        const thread = this._threads.get(args.threadId);
        if (thread) {
            this.sendEvent(new vscode_debugadapter_1.ContinuedEvent(args.threadId, false));
            this._debugManager.resume(thread.serverThreadId);
        }
    }
    nextRequest(response, args) {
        const thread = this._threads.get(args.threadId);
        if (thread) {
            const threadId = thread.serverThreadId;
            this._debugManager.stepOver(threadId);
            this.sendResponse(response);
        }
    }
    stepInRequest(response, args) {
        const thread = this._threads.get(args.threadId);
        if (thread) {
            const threadId = thread.serverThreadId;
            this._debugManager.stepIn(threadId);
            this.sendResponse(response);
        }
    }
    stepOutRequest(response, args) {
        const thread = this._threads.get(args.threadId);
        if (thread) {
            const threadId = thread.serverThreadId;
            this._debugManager.stepOut(threadId);
            this.sendResponse(response);
        }
    }
    disconnectRequest(response, args) {
        if (this._debugServer) {
            if (IS_WIN) {
                ms_wmic_1.execute("process where \"Commandline like '%org.ballerinalang.launcher.Main%'\" CALL TERMINATE", () => {
                    this._debugManager.kill();
                    this._debugServer.kill();
                });
            }
            else {
                ps_node_1.lookup({
                    arguments: ['org.ballerinalang.launcher.Main', ...this._executableArgs],
                }, (err, resultList) => {
                    resultList.forEach((process) => {
                        ps_node_1.kill(process.pid);
                    });
                    this._debugManager.kill();
                    this._debugServer.kill();
                });
            }
        }
        this.sendResponse(response);
    }
    terminate(msg) {
        this.sendEvent(new vscode_debugadapter_1.OutputEvent(msg));
        this.sendEvent(new vscode_debugadapter_1.TerminatedEvent());
    }
}
exports.BallerinaDebugSession = BallerinaDebugSession;
//# sourceMappingURL=session.js.map