"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright (c) 2018, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */
const vscode_1 = require("vscode");
const _ = require("lodash");
const renderer_1 = require("./renderer");
const utils_1 = require("../utils");
const path_1 = require("path");
const DEBOUNCE_WAIT = 500;
let diagramViewPanel;
let activeEditor;
let preventDiagramUpdate = false;
let rpcHandler;
function updateWebView(docUri) {
    if (rpcHandler) {
        rpcHandler.invokeRemoteMethod("updateAST", [docUri.toString()], () => { });
    }
}
function showDiagramEditor(context, langClient) {
    const didChangeDisposable = vscode_1.workspace.onDidChangeTextDocument(_.debounce((e) => {
        if (activeEditor && (e.document === activeEditor.document) &&
            e.document.fileName.endsWith('.bal')) {
            if (preventDiagramUpdate) {
                return;
            }
            updateWebView(e.document.uri);
        }
    }, DEBOUNCE_WAIT));
    const changeActiveEditorDisposable = vscode_1.window.onDidChangeActiveTextEditor((activatedEditor) => {
        if (vscode_1.window.activeTextEditor && activatedEditor
            && (activatedEditor.document === vscode_1.window.activeTextEditor.document)
            && activatedEditor.document.fileName.endsWith('.bal')) {
            activeEditor = vscode_1.window.activeTextEditor;
            updateWebView(activatedEditor.document.uri);
        }
    });
    if (diagramViewPanel) {
        diagramViewPanel.reveal(vscode_1.ViewColumn.Two, true);
        return;
    }
    // Create and show a new webview
    diagramViewPanel = vscode_1.window.createWebviewPanel('ballerinaDiagram', "Ballerina Diagram", { viewColumn: vscode_1.ViewColumn.Two, preserveFocus: true }, {
        enableScripts: true,
        retainContextWhenHidden: true,
    });
    diagramViewPanel.iconPath = {
        light: vscode_1.Uri.file(path_1.join(context.extensionPath, 'resources/images/icons/design-view.svg')),
        dark: vscode_1.Uri.file(path_1.join(context.extensionPath, 'resources/images/icons/design-view-inverse.svg'))
    };
    const editor = vscode_1.window.activeTextEditor;
    if (!editor) {
        return;
    }
    activeEditor = editor;
    rpcHandler = utils_1.WebViewRPCHandler.create(diagramViewPanel.webview, langClient);
    const html = renderer_1.render(context, langClient, editor.document.uri);
    if (diagramViewPanel && html) {
        diagramViewPanel.webview.html = html;
    }
    diagramViewPanel.onDidDispose(() => {
        diagramViewPanel = undefined;
        didChangeDisposable.dispose();
        changeActiveEditorDisposable.dispose();
    });
}
function activate(ballerinaExtInstance) {
    let context = ballerinaExtInstance.context;
    let langClient = ballerinaExtInstance.langClient;
    const diagramRenderDisposable = vscode_1.commands.registerCommand('ballerina.showDiagram', () => {
        return ballerinaExtInstance.onReady()
            .then(() => {
            const { experimental } = langClient.initializeResult.capabilities;
            const serverProvidesAST = experimental && experimental.astProvider;
            if (!serverProvidesAST) {
                ballerinaExtInstance.showMessageServerMissingCapability();
                return {};
            }
            showDiagramEditor(context, langClient);
        })
            .catch((e) => {
            if (!ballerinaExtInstance.isValidBallerinaHome()) {
                ballerinaExtInstance.showMessageInvalidBallerinaHome();
            }
            else {
                ballerinaExtInstance.showPluginActivationError();
            }
        });
    });
    ballerinaExtInstance.onReady().then(() => {
        const args = {
            settings: vscode_1.workspace.getConfiguration('ballerina'),
        };
        langClient.sendNotification("workspace/didChangeConfiguration", args);
        langClient.onNotification('window/showTextDocument', (location) => {
            if (location.uri !== undefined) {
                vscode_1.window.showTextDocument(vscode_1.Uri.parse(location.uri.toString()), { selection: location.range });
            }
        });
    })
        .catch((e) => {
        vscode_1.window.showErrorMessage('Could not start openFile capability listener', e.message);
    });
    context.subscriptions.push(diagramRenderDisposable);
}
exports.activate = activate;
//# sourceMappingURL=activator.js.map