"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright (c) 2018, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */
const vscode_1 = require("vscode");
const _ = require("lodash");
const renderer_1 = require("./renderer");
const DEBOUNCE_WAIT = 500;
let previewPanel;
let activeEditor;
function updateWebView(ast, docUri) {
    if (previewPanel) {
        previewPanel.webview.postMessage({
            command: 'update',
            json: ast,
            docUri: docUri.toString()
        });
    }
}
function showDocs(context, langClient, nodeName) {
    const didChangeDisposable = vscode_1.workspace.onDidChangeTextDocument(_.debounce((e) => {
        if (activeEditor && (e.document === activeEditor.document) &&
            e.document.fileName.endsWith('.bal')) {
            const docUri = e.document.uri;
            langClient.getAST(docUri)
                .then((resp) => {
                if (resp.ast) {
                    updateWebView(resp.ast, docUri);
                }
            });
        }
    }, DEBOUNCE_WAIT));
    const changeActiveEditorDisposable = vscode_1.window.onDidChangeActiveTextEditor((activatedEditor) => {
        if (vscode_1.window.activeTextEditor && activatedEditor
            && (activatedEditor.document === vscode_1.window.activeTextEditor.document)
            && activatedEditor.document.fileName.endsWith('.bal')) {
            activeEditor = vscode_1.window.activeTextEditor;
            const docUri = activatedEditor.document.uri;
            langClient.getAST(docUri)
                .then((resp) => {
                if (resp.ast) {
                    updateWebView(resp.ast, docUri);
                }
            });
        }
    });
    if (previewPanel) {
        previewPanel.reveal(vscode_1.ViewColumn.Two, true);
        scrollToTitle(previewPanel, nodeName);
        return;
    }
    // Create and show a new webview
    previewPanel = vscode_1.window.createWebviewPanel('ballerinaDocs', "Ballerina Docs", { viewColumn: vscode_1.ViewColumn.Two, preserveFocus: true }, {
        enableScripts: true,
        retainContextWhenHidden: true,
    });
    const editor = vscode_1.window.activeTextEditor;
    if (!editor) {
        return;
    }
    activeEditor = editor;
    const html = renderer_1.render(context, langClient);
    if (previewPanel && html) {
        previewPanel.webview.html = html;
    }
    const disposeLoaded = previewPanel.webview.onDidReceiveMessage((e) => {
        if (e.message !== "loaded-doc-preview") {
            return;
        }
        disposeLoaded.dispose();
        langClient.getAST(editor.document.uri)
            .then((resp) => {
            if (resp.ast) {
                updateWebView(resp.ast, editor.document.uri);
                if (previewPanel) {
                    scrollToTitle(previewPanel, nodeName);
                }
            }
        });
    });
    previewPanel.onDidDispose(() => {
        previewPanel = undefined;
        didChangeDisposable.dispose();
        changeActiveEditorDisposable.dispose();
    });
}
function activate(ballerinaExtInstance) {
    let context = ballerinaExtInstance.context;
    let langClient = ballerinaExtInstance.langClient;
    const docsRenderDisposable = vscode_1.commands.registerCommand('ballerina.showDocs', nodeNameArg => {
        return ballerinaExtInstance.onReady()
            .then(() => {
            const { experimental } = langClient.initializeResult.capabilities;
            const serverProvidesAST = experimental && experimental.astProvider;
            if (!serverProvidesAST) {
                ballerinaExtInstance.showMessageServerMissingCapability();
                return {};
            }
            showDocs(context, langClient, (nodeNameArg) ? nodeNameArg.argumentV : "");
        })
            .catch((e) => {
            if (!ballerinaExtInstance.isValidBallerinaHome()) {
                ballerinaExtInstance.showMessageInvalidBallerinaHome();
            }
            else {
                ballerinaExtInstance.showPluginActivationError();
            }
        });
    });
    context.subscriptions.push(docsRenderDisposable);
}
exports.activate = activate;
function scrollToTitle(previewPanel, anchorId) {
    if (previewPanel && anchorId) {
        previewPanel.webview.postMessage({
            command: 'scroll',
            anchor: anchorId
        });
    }
}
//# sourceMappingURL=activator.js.map