"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
const getLangClientMethods = (langClient) => {
    return [{
            methodName: 'getAST',
            handler: (args) => {
                return langClient.onReady().then(() => {
                    return langClient.getAST(args[0]);
                });
            }
        },
        {
            methodName: 'astDidChange',
            handler: (args) => {
                return langClient.onReady().then(() => {
                    return langClient.triggerASTDidChange(args[0], args[1]);
                });
            }
        },
        {
            methodName: 'getEndpoints',
            handler: (args) => {
                return langClient.onReady().then(() => {
                    return langClient.getEndpoints();
                });
            }
        },
        {
            methodName: 'parseFragment',
            handler: (args) => {
                return langClient.onReady().then(() => {
                    return langClient.parseFragment({
                        enclosingScope: args[0].enclosingScope,
                        expectedNodeType: args[0].expectedNodeType,
                        source: args[0].source
                    });
                });
            }
        },
        {
            methodName: 'revealRange',
            handler: (args) => {
                const params = JSON.parse(args[0]);
                const revealRangeInEditor = (editor) => {
                    const { start, end } = params.range;
                    const startPosition = new vscode_1.Position(start.line - 1, start.character - 1);
                    const endPosition = new vscode_1.Position(end.line - 1, end.character - 1);
                    editor.revealRange(new vscode_1.Range(startPosition, endPosition));
                    editor.selection = new vscode_1.Selection(startPosition, endPosition);
                };
                const activeTextEditor = vscode_1.window.activeTextEditor;
                const visibleTextEditors = vscode_1.window.visibleTextEditors;
                const findByDocUri = (editor) => editor.document.uri.toString()
                    === params.textDocumentIdentifier.uri;
                const foundVisibleEditor = visibleTextEditors.find(findByDocUri);
                if (activeTextEditor && findByDocUri(activeTextEditor)) {
                    revealRangeInEditor(activeTextEditor);
                }
                else if (foundVisibleEditor) {
                    revealRangeInEditor(foundVisibleEditor);
                    return Promise.resolve();
                }
                else {
                    return vscode_1.window.showTextDocument(vscode_1.Uri.parse(params.textDocumentIdentifier.uri), {
                        viewColumn: vscode_1.ViewColumn.One
                    })
                        .then((textEditor) => {
                        revealRangeInEditor(textEditor);
                    });
                }
            }
        },
        {
            methodName: 'goToSource',
            handler: (args) => {
                const activeEditor = vscode_1.window.activeTextEditor;
                if (activeEditor) {
                    // TODO
                }
                return Promise.resolve();
            }
        },
        {
            methodName: 'getExamples',
            handler: (args) => {
                return langClient.onReady().then(() => {
                    return langClient.fetchExamples();
                });
            }
        },
        {
            methodName: 'getDefinitionPosition',
            handler: (args) => {
                return langClient.getDefinitionPosition(args[0]);
            }
        }
    ];
};
const undoRedoMethods = [{
        methodName: 'undo',
        handler: (args) => {
            vscode_1.commands.executeCommand('workbench.action.focusPreviousGroup')
                .then(() => {
                vscode_1.commands.executeCommand('undo');
            });
        }
    },
    {
        methodName: 'redo',
        handler: (args) => {
            vscode_1.commands.executeCommand('workbench.action.focusPreviousGroup')
                .then(() => {
                vscode_1.commands.executeCommand('redo');
            });
        }
    }
];
class WebViewRPCHandler {
    constructor(methods, webView) {
        this.methods = methods;
        this.webView = webView;
        this._sequence = 1;
        this._callbacks = new Map();
        webView.onDidReceiveMessage(this._onRemoteMessage.bind(this));
    }
    _getMethod(methodName) {
        return this.methods.find(method => (method.methodName === methodName));
    }
    _onRemoteMessage(msg) {
        if (msg.id !== undefined) {
            // this is a request from remote
            const method = this._getMethod(msg.methodName);
            if (method) {
                method.handler(msg.arguments || [])
                    .then((response) => {
                    this.webView.postMessage({
                        originId: msg.id,
                        response,
                    });
                });
            }
        }
        else if (msg.originId !== undefined) {
            // this is a response from remote to one of our requests
            const callback = this._callbacks.get(msg.originId);
            if (callback) {
                callback(msg.response);
                this._callbacks.delete(msg.originId);
            }
        }
    }
    invokeRemoteMethod(methodName, args = [], callback) {
        const msg = {
            id: this._sequence,
            methodName,
            arguments: args,
        };
        this._callbacks.set(this._sequence, callback);
        this.webView.postMessage(msg);
        this._sequence++;
    }
    static create(webView, langClient, methods = []) {
        return new WebViewRPCHandler([...methods, ...getLangClientMethods(langClient), ...undoRedoMethods], webView);
    }
    dispose() {
        // TODO unregister event handlers
    }
}
exports.WebViewRPCHandler = WebViewRPCHandler;
//# sourceMappingURL=handler.js.map